"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("../");
const fs_1 = require("fs");
const printer_1 = __importDefault(require("./printer"));
const utils_1 = require("./utils");
beforeEach(() => {
    jest.spyOn(console, 'log').mockImplementation();
    jest.spyOn(console, 'warn').mockImplementation();
});
afterEach(() => {
    jest.clearAllMocks();
    __1.clearAllUDFHelpers();
});
describe('addUDFHelper', () => {
    describe('basic', () => {
        const type = 'basic';
        const dirs = ['declaration', 'dependencies'];
        for (const dir of dirs) {
            test(dir, () => {
                const helpers = require(utils_1.inputFixturePath([type, dir]));
                // @ts-ignore
                const programFuncs = [(pass) => pass.addUDFHelper(dir)];
                const code = printer_1.default({ helpers, programFuncs });
                expect(code).toMatchSnapshot();
                fs_1.writeFile(utils_1.outputFixturePath([type, dir]), code || '', () => { });
            });
        }
        test('rename', () => {
            const dir = 'rename';
            const helpers = require(utils_1.inputFixturePath([type, dir]));
            const content = `function _rename() { return "path.replaceWith" }`;
            const programFuncs = [(pass) => pass.addUDFHelper(dir)];
            // @ts-ignore
            const code = printer_1.default({ content, helpers, programFuncs });
            expect(code).toMatchSnapshot();
            fs_1.writeFile(utils_1.outputFixturePath([type, dir]), code || '', () => { });
        });
        test('multi plugins', () => {
            const dir = 'dependencies';
            const helpers = require(utils_1.inputFixturePath([type, dir]));
            const programFuncs = [(pass) => pass.addUDFHelper(dir), (pass) => pass.addUDFHelper(dir)];
            // @ts-ignore
            const code = printer_1.default({ helpers, programFuncs });
            expect(code).toMatchSnapshot();
            fs_1.writeFile(utils_1.outputFixturePath([type, dir]), code || '', () => { });
        });
    });
});
describe('listUDFHelper', () => {
    describe('basic', () => {
        const type = 'basic';
        test('officialMix', () => {
            const dir = 'officialMix';
            const helpers = require(utils_1.inputFixturePath([type, dir]));
            const programFuncs = [(pass) => console.log(pass.listUDFHelper())];
            // @ts-ignore
            printer_1.default({ helpers, programFuncs });
            expect(console.log.mock.calls[0][0]).toMatchSnapshot();
        });
    });
});
//# sourceMappingURL=basic.test.js.map