"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("../");
const errors_1 = require("./../errors");
const __2 = require("../");
const printer_1 = __importDefault(require("./printer"));
const utils_1 = require("./utils");
beforeEach(() => {
    jest.spyOn(console, 'log').mockImplementation();
    jest.spyOn(console, 'warn').mockImplementation();
});
afterEach(() => {
    jest.clearAllMocks();
    __2.clearAllUDFHelpers();
});
describe('addUDFHelper', () => {
    describe('error', () => {
        const type = 'error';
        describe('core_ext', () => {
            describe('babelImplemented', () => {
                // prettier-ignore
                const cases = [
                    {
                        method: 'BabelFile#addUDFHelper',
                        preFunc: (pass, helpers) => {
                            try {
                                pass.file.constructor.prototype.addUDFHelper = function () { };
                                __1.useDangerousUDFHelpers(pass, helpers);
                            }
                            catch (e) {
                                pass.file.constructor.prototype.addUDFHelper = undefined;
                                throw e;
                            }
                        }
                    },
                    {
                        method: 'BabelFile#listUDFHelper',
                        preFunc: (pass, helpers) => {
                            try {
                                pass.file.constructor.prototype.listUDFHelper = function () { };
                                __1.useDangerousUDFHelpers(pass, helpers);
                            }
                            catch (e) {
                                pass.file.constructor.prototype.listUDFHelper = undefined;
                                throw e;
                            }
                        }
                    },
                ];
                for (const c of cases) {
                    const { method, preFunc } = c;
                    test(method, () => {
                        const dir = 'declaration';
                        const helpers = require(utils_1.inputFixturePath(['basic', dir]));
                        const programFuncs = [(pass) => pass.addUDFHelper('declaration')];
                        // prettier-ignore
                        expect(() => { printer_1.default({ helpers, programFuncs, preFunc }); }).toThrowError(new errors_1.AlreadyImplementedError(`unknown:\
 \nThis tool cannot be used. officially supported.
Please see the official documentation.
https://babeljs.io/docs/en/babel-helpers
`));
                        // prettier-ignore
                        expect(() => { printer_1.default({ helpers, programFuncs, preFunc }); }).toThrowErrorMatchingSnapshot();
                    });
                }
            });
            test('Not found UDF helpers', () => {
                const programFuncs = [(pass) => pass.addUDFHelper('objectWithoutProperties')];
                // prettier-ignore
                expect(() => { printer_1.default({ programFuncs }); }).toThrowError(new errors_1.NotFoundError('unknown: Not found UDF helpers.'));
                // prettier-ignore
                expect(() => { printer_1.default({ programFuncs }); }).toThrowErrorMatchingSnapshot();
            });
            test('babelAlreadyDefined', () => {
                const dir = 'babelAlreadyDefined';
                const helpers = require(utils_1.inputFixturePath([type, dir]));
                const programFuncs = [(pass) => pass.addUDFHelper('objectWithoutProperties')];
                // prettier-ignore
                expect(() => { printer_1.default({ helpers, programFuncs }); }).toThrowError(new errors_1.AlreadyImplementedError('unknown: objectWithoutProperties cannot be used because it is supported by babel official.\nPlease change the name of the helper.'));
                // prettier-ignore
                expect(() => { printer_1.default({ helpers, programFuncs }); }).toThrowErrorMatchingSnapshot();
            });
        });
        describe('DependencyResolvePlugin', () => {
            // prettier-ignore
            const cases = [
                { dir: 'doNotExist', err: new ReferenceError('unknown: Unknown helper doNotExist') },
                { dir: 'importNamed', err: new SyntaxError('unknown: UDF Helpers can only import a default value (This is an error on an internal node. Probably an internal error.)') },
                { dir: 'doNotGiveName', err: new SyntaxError('unknown: UDF Helpers should give names to their exported func declaration (This is an error on an internal node. Probably an internal error.)') },
                { dir: 'expression', err: new SyntaxError('unknown: UDF Helpers must be a function declaration (This is an error on an internal node. Probably an internal error.)') },
                { dir: 'exportAll', err: new SyntaxError('unknown: UDF Helpers can only export default (This is an error on an internal node. Probably an internal error.)') },
                { dir: 'exportNamed', err: new SyntaxError('unknown: UDF Helpers can only export default (This is an error on an internal node. Probably an internal error.)') },
                { dir: 'doNotExportDefault', err: new SyntaxError('unknown: UDF Helpers must default-export something.') },
            ];
            for (const c of cases) {
                const { dir, err } = c;
                test(dir, () => {
                    const helpers = require(utils_1.inputFixturePath([type, dir]));
                    const programFuncs = [(pass) => pass.addUDFHelper(dir)];
                    // prettier-ignore
                    expect(() => { printer_1.default({ helpers, programFuncs }); }).toThrowError(err);
                    // prettier-ignore
                    expect(() => { printer_1.default({ helpers, programFuncs }); }).toThrowErrorMatchingSnapshot();
                });
            }
        });
    });
});
//# sourceMappingURL=error.test.js.map