"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensure = exports.buildProgram = exports.getDependencies = void 0;
const t = __importStar(require("@babel/types"));
const store_1 = require("../store");
const plugins_1 = require("./plugins");
// @ts-ignore
let fileClass = undefined;
function createBuilderFile(helper) {
    if (fileClass) {
        const options = {
            iGlobals: new Set(),
            iLocalBindingNames: new Set(),
            iDependencies: new Map(),
            iImportBindingsReferences: new Array(),
            iImportPaths: new Array(),
            iExportBindingAssignments: new Array(),
            iExportName: undefined,
            iExportPath: undefined,
        };
        const params = { ast: t.file(helper.ast()), code: '' };
        // @ts-ignore
        const file = new fileClass(options, params);
        return file;
    }
}
// MEMO:
// The helperDataCache cannot be hoisted, so the valid objects below are
const helperDataCache = Object.create(null);
function loadHelper(name) {
    if (!helperDataCache[name]) {
        const helper = store_1.helpers[name];
        if (!helper) {
            throw Object.assign(new ReferenceError(`Unknown helper ${name}`), {
                code: 'BABEL_UDF_HELPER_UNKNOWN',
                helper: name,
            });
        }
        const builderFile = createBuilderFile(helper);
        // Traverse import statements and build global variable dependencies
        plugins_1.DependencyResolvePlugin(builderFile);
        plugins_1.ReferencedResolvePlugin(builderFile);
        const opts = builderFile.opts;
        helperDataCache[name] = {
            build: ({ currentId, currentDependencies, currentLocalBindingNames }) => {
                /**
                 *
                 * A new helperBuilder instance needs to be created for each traverse,
                 * but only opts needs to be inherited.
                 *
                 * https://github.com/babel/babel/blob/c664fbdd07d0a510d5bcb42b4d1776e9354696ad/packages/babel-helpers/src/index.js#L245-L263
                 */
                const builderFile = createBuilderFile(helper);
                builderFile.opts = opts;
                /**
                 * Remove helper export default.
                 * ・Remove helper import.
                 * ・Resolve Referenced Identifier. (rename)
                 * ・Global variable traversal (globals)
                 * ・Helper code to actually write (nodes)
                 */
                plugins_1.ProgramBuildPlugin(builderFile, {
                    currentId,
                    currentDependencies,
                    currentLocalBindingNames,
                });
                return {
                    nodes: builderFile.ast.program.body,
                    // @ts-ignore
                    globals: builderFile.opts['iGlobals'],
                };
            },
            // @ts-ignore
            dependencies: () => builderFile.opts['iDependencies'],
        };
    }
    return helperDataCache[name];
}
function getDependencies(name) {
    return loadHelper(name).dependencies().values();
}
exports.getDependencies = getDependencies;
function buildProgram({ name, currentId, currentDependencies, currentLocalBindingNames, }) {
    return loadHelper(name).build({ currentId, currentDependencies, currentLocalBindingNames });
}
exports.buildProgram = buildProgram;
/**
 * MEMO:
 * Initialize
 *  ・Setting the builder class to build the helper.
 *  ・Setting dependencies (global) when renaming etc. are not considered.
 */
function ensure(name, builderFileClass) {
    // @ts-ignore
    if (!fileClass)
        fileClass = builderFileClass;
    loadHelper(name);
}
exports.ensure = ensure;
//# sourceMappingURL=index.js.map