"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const traverse_1 = __importDefault(require("@babel/traverse"));
const utils_1 = require("./utils");
const store_1 = require("../../store");
function DependencyResolve(file, options) {
    const { opts } = file;
    const { iDependencies, iImportPaths } = opts;
    const visitor = {
        ImportDeclaration(path) {
            // @ts-ignore
            const name = path.node.source.value;
            if (!store_1.helpers[name]) {
                throw new ReferenceError(`Unknown UDF helper ${name}`);
            }
            if (
            // @ts-ignore
            path.get('specifiers').length !== 1 ||
                // @ts-ignore
                !path.get('specifiers.0').isImportDefaultSpecifier()) {
                throw path.buildCodeFrameError('UDF Helpers can only import a default value');
            }
            // @ts-ignore
            const bindingIdentifier = path.node.specifiers[0].local;
            // STEP 1: Specify dependencies
            iDependencies.set(bindingIdentifier, name);
            // STEP 2: Specify import path
            iImportPaths.push(utils_1.makePath(path));
        },
        ExportDefaultDeclaration(path) {
            const declar = path.get('declaration');
            // MEMO:
            // Error if not a named function declaration
            // @ts-ignore
            if (declar.isFunctionDeclaration()) {
                // @ts-ignore
                if (!declar.node.id) {
                    // @ts-ignore
                    throw declar.buildCodeFrameError('UDF Helpers should give names to their exported func declaration');
                }
                // STEP 3: Each helper has only one export default name. keep its name
                // @ts-ignore
                opts['iExportName'] = declar.node.id.name;
            }
            else {
                // @ts-ignore
                throw declar.buildCodeFrameError('UDF Helpers must be a function declaration');
            }
            // STEP 4: Each helper has only one export default path. keep its name
            // @ts-ignore
            opts['iExportPath'] = utils_1.makePath(path);
        },
        ExportAllDeclaration(path) {
            throw path.buildCodeFrameError('UDF Helpers can only export default');
        },
        ExportNamedDeclaration(path) {
            throw path.buildCodeFrameError('UDF Helpers can only export default');
        },
        Statement(path) {
            if (path.isModuleDeclaration())
                return;
            path.skip();
        },
    };
    traverse_1.default(file.ast, visitor, file.scope);
    if (!opts['iExportName'])
        throw new SyntaxError('UDF Helpers must default-export something.');
}
exports.default = DependencyResolve;
//# sourceMappingURL=DependencyResolve.js.map