"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const t = __importStar(require("@babel/types"));
const traverse_1 = __importDefault(require("@babel/traverse"));
function ProgramBuild(file, options) {
    const { opts } = file;
    const { iDependencies, iLocalBindingNames, iImportPaths, iImportBindingsReferences, iExportPath, iExportName, iExportBindingAssignments, } = opts;
    const { currentId, currentDependencies, currentLocalBindingNames } = options;
    if (currentLocalBindingNames && !currentId)
        throw new Error('Unexpected local bindings for module-based UDF helpers.');
    if (!currentId)
        return;
    // STEP1: Update dependency
    const updatedDependencies = {};
    iDependencies.forEach((name, id) => {
        updatedDependencies[id.name] = currentDependencies[name] || currentId;
    });
    const renames = {};
    const cBindingNameSet = new Set(currentLocalBindingNames || []);
    iLocalBindingNames.forEach((name) => {
        let newName = name;
        while (cBindingNameSet.has(newName))
            newName = `_${newName}`;
        // STEP2: Identifying the name change target
        if (newName !== name)
            renames[name] = newName;
    });
    if (currentId.type === 'Identifier' && iExportName !== currentId.name) {
        renames[iExportName] = currentId.name;
    }
    const visitor = {
        Program(path) {
            const exportNode = path.get(iExportPath);
            const importPaths = iImportPaths.map((p) => path.get(p));
            const importBindingRefs = iImportBindingsReferences.map((p) => path.get(p));
            const declar = exportNode.get('declaration');
            if (currentId.type === 'Identifier') {
                if (declar.isFunctionDeclaration()) {
                    /**
                     * STEP 3.1: Remove export default (Function Declaration)
                     *
                     * helper`export default function fd(){}`
                     * ↓
                     * function fd(){}
                     *
                     */
                    exportNode.replaceWith(declar);
                }
                else {
                    /**
                     * STEP 3.2: Remove export defualt (Function Expression)
                     *
                     * helper`
                     * var fe = function fe(){}
                     * export default fe
                     * `
                     * ↓
                     * var fe = function fe(){}
                     */
                    exportNode.replaceWith(t.variableDeclaration('var', [t.variableDeclarator(currentId, declar['node'])]));
                }
            }
            else if (currentId.type === 'MemberExpression') {
                /**
                 * MEMO:
                 *
                 *　↓↓↓ I can't think of an example where processing goes below this ↓↓↓
                 *　It is written in the code on the babel side, and it is written because it seems to be meaningful code
                 *
                 *  https://github.com/babel/babel/blob/3d498d05e737b6b497df55a177c113fd8167b744/packages/babel-helpers/src/index.js#L194-L212
                 */
                if (declar.isFunctionDeclaration()) {
                    iExportBindingAssignments.forEeach((assignPath) => {
                        const assign = path.get(assignPath);
                        assign.replaceWith(t.assignmentExpression('=', currentId, assign['node']));
                    });
                    exportNode.replaceWith(declar);
                    /**
                     * Cannot resolve pushContainer type
                     * https://github.com/DefinitelyTyped/DefinitelyTyped/commit/248de54338ef00c66f904aa06c568893d4692f16#diff-a73ef04a9efc8d29b305baa51072da24R447
                     *
                     */
                    path.pushContainer(
                    // @ts-ignore
                    'body', t.expressionStatement(t.assignmentExpression('=', currentId, t.identifier(iExportName))));
                }
                else {
                    exportNode.replaceWith(t.expressionStatement(t.assignmentExpression('=', currentId, declar['node'])));
                }
            }
            else {
                throw new Error('Unexpected UDF helper format.');
            }
            Object.keys(renames).forEach((name) => {
                path.scope.rename(name, renames[name]);
            });
            // STEP 4: Remove imports from helpers
            for (const path of importPaths)
                path.remove();
            for (const path of importBindingRefs) {
                const id = updatedDependencies[path['node'].name];
                const node = t.cloneNode(id);
                // STEP 5: Rename import binding reference
                path.replaceWith(node);
            }
            // STEP 6: Stop traversing for helpers
            path.stop();
        },
    };
    // MEMO
    // Workaround Error: Argument of type '{ Program(path: NodePath<Node>): void; }' is not assignable to parameter of type 'TraverseOptions<Node>'.
    traverse_1.default(file.ast, visitor, file.scope);
}
exports.default = ProgramBuild;
//# sourceMappingURL=ProgramBuild.js.map