"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.listUDFHelper = exports.addUDFHelper = void 0;
const t = __importStar(require("@babel/types"));
const builder = __importStar(require("../builder"));
const errors_1 = require("../errors");
const store_1 = require("../store");
// MEMO:
// binding to an instance of BabelFile
function addUDFHelper(name) {
    let isAlreadyExist;
    /**
     * MEMO:
     *
     * availableHelper may return an error.
     * https://github.com/babel/babel/blob/3d498d05e737b6b497df55a177c113fd8167b744/packages/babel-core/src/transformation/file/file.js#L134
     */
    try {
        isAlreadyExist = this.availableHelper(name, undefined);
    }
    catch (e) {
        throw e;
    }
    if (isAlreadyExist)
        throw new errors_1.AlreadyImplementedError(`${name} cannot be used because it is supported by babel official.\nPlease change the name of the helper.`);
    const declar = this.declarations[name];
    if (declar)
        return t.cloneNode(declar);
    // make sure that the helper exists
    // this.constructor is babel.BabelFile class
    builder.ensure(name, this.constructor);
    const uid = (this.declarations[name] = this.scope.generateUidIdentifier(name));
    const dependencies = {};
    for (const dep of builder.getDependencies(name)) {
        dependencies[dep] = this.addUDFHelper(dep);
    }
    const { nodes, globals } = builder.buildProgram({
        name,
        currentId: uid,
        currentDependencies: dependencies,
        currentLocalBindingNames: Object.keys(this.scope.getAllBindings()),
    });
    // MEMO:
    // Rename if global variables overlap
    globals.forEach((name) => {
        if (this.path.scope.hasBinding(name, true /* noGlobals */)) {
            this.path.scope.rename(name);
        }
    });
    // MEMO:
    // babel/generator reads _compact and compacts the output
    // https://github.com/babel/babel/blob/379e1c55937231de15ca97b475942b96983aa330/packages/babel-generator/src/printer.js#L374
    nodes.forEach((node) => {
        node._compact = true;
    });
    this.path.unshiftContainer('body', nodes);
    // MEMO:
    // NodePath# is not automatically registered in bindings, so the following code is required
    this.path.get('body').forEach((path) => {
        if (nodes.indexOf(path.node) === -1)
            return;
        if (path.isVariableDeclaration())
            this.scope.registerDeclaration(path);
    });
    return uid;
}
exports.addUDFHelper = addUDFHelper;
// MEMO:
// binding to an instance of BabelFile
function listUDFHelper() {
    return Object.keys(store_1.helpers).reduce((acc, name) => {
        let isAlreadyExist;
        /**
         * MEMO:
         *
         * availableHelper may return an error.
         * https://github.com/babel/babel/blob/3d498d05e737b6b497df55a177c113fd8167b744/packages/babel-core/src/transformation/file/file.js#L134
         */
        try {
            isAlreadyExist = this.availableHelper(name, undefined);
        }
        catch (e) {
            throw e;
        }
        if (isAlreadyExist) {
            acc.unavailable.push(name);
        }
        else {
            acc.available.push(name);
        }
        return acc;
    }, { available: [], unavailable: [] });
}
exports.listUDFHelper = listUDFHelper;
//# sourceMappingURL=file.js.map